function [y, e] = filtrNLMS(M,L,d,x,mi,gamma,alfa,delta,leaky,ialg)
% Przyklad 7.1
% Autor: Tomasz Zielinski, tzielin@agh.edu.pl

bx = zeros(M,1);       % inicjowanie bufora bx sygnalu wejsciowego x[n]
h = zeros(M,1);        % inicjowanie wag filtra
coef = 1-mi*leaky;     % wspolczynnik pamietania wag h, zwykle 1, jesli leaky=0
y = []; e = [];        % wyzerowanie sygnalow wyjsciowych
Rinv = delta*eye(M,M); % LMS-N: inicjowanie odwrotnosci macierzy autokorelacji sygnalu x[n]
X = zeros(L,M);        % AP-NLMS: macierz na probki x[n]
bd = zeros(L,1);       % AP-NLMS: inicjowanie bufora bd sygnalu wejsciowego d[n]

for n = 1 : length(x)
    if(n==1) bxx = eps*ones(M,1); else bxx=bx; end  % zapamietaj poprzednia wartosc
    bx = [ x(n); bx(1:M-1) ];                       % pobierz nowa probke x[n] do bufora
    if(ialg==3)   % algorytm AP-NLMS
       X = [ bx'; X(1:L-1,1:M) ];   % pobierz nowy wiersz do macierzy X
       bd = [ d(n); bd(1:L-1) ];    % pobierz nowa probke d[n] do bufora bd
       yL = X*h; y(n) = yL(1);      % oblicz yL, y[n]
       eL = bd - yL; e(n)=eL(1);    % oblicz eL, e[n]
    else          % algorytmy POZOSTALE
       y(n) = h' * bx;              % oblicz y[n] = sum( x .* bx)
       e(n) = d(n) - y(n);          % oblicz e[n]
    end
    if (ialg==1)  % LMS
        h = coef*h + mi * e(n) * bx;
    end 
    if (ialg==2)  % NLMS
       energia = bx' * bx; 
       h = coef*h + mi/(gamma+energia) * e(n) * bx;       % modyfikacja wag
    end
    if (ialg==3)  % AP-NLMS
       h = coef*h + mi* X'*inv( X*X'+gamma*eye(L) )*eL;   % modyfikacja wag
    end
    if (ialg==4)  % zdekorelowany LMS
       alfa = (bx'*bxx) / (bxx'*bxx);
       v = bx - alfa*bxx; %
       h = coef*h + mi * e(n) * v;                        % modyfikacja wag
    end
    if (ialg==5)  % zdekorelowany NLMS
       alfa = (bx'*bxx) / (bxx'*bxx);                     
       v = bx - alfa*bxx; %
       h = coef*h + mi/(gamma+bx'*bx) * e(n) * v;         % modyfikacja wag wer. 1
     % h = coef*h + mi/(gamma+bx'*v) * e(n) * v;          % modyfikacja wag wer.2
    end
    if (ialg==6) % filtr LMS-N (Newton); ponizej modyfikacja Rinv
       Rinv = (Rinv - Rinv*bx*bx'*Rinv / ((1-alfa)/alfa+bx'*Rinv*bx)) / (1-alfa);
       h = coef*h + mi*Rinv * bx * e(n);                  % modyfikacja wag
    end
end % of for